@@ -4,19 +4,22 @@ Support for prepared statements
 
 # Copyright (C) 2020 The Psycopg Team
 
+from __future__ import annotations
+
 from enum import IntEnum, auto
-from typing import Iterator, Optional, Sequence, Tuple, TYPE_CHECKING
+from typing import Any, Sequence, TYPE_CHECKING
 from collections import OrderedDict
-from typing_extensions import TypeAlias
 
 from . import pq
-from ._compat import Deque
+from .abc import PQGen
+from ._compat import Deque, TypeAlias
 from ._queries import PostgresQuery
 
 if TYPE_CHECKING:
     from .pq.abc import PGresult
+    from ._connection_base import BaseConnection
 
-Key: TypeAlias = Tuple[bytes, Tuple[int, ...]]
+Key: TypeAlias = "tuple[bytes, tuple[int, ...]]"
 
 COMMAND_OK = pq.ExecStatus.COMMAND_OK
 TUPLES_OK = pq.ExecStatus.TUPLES_OK
@@ -30,7 +33,7 @@ class Prepare(IntEnum):
 
 class PrepareManager:
     # Number of times a query is executed before it is prepared.
-    prepare_threshold: Optional[int] = 5
+    prepare_threshold: int | None = 5
 
     # Maximum number of prepared statements on the connection.
     prepared_max: int = 100
@@ -45,15 +48,15 @@ class PrepareManager:
         # Counter to generate prepared statements names
         self._prepared_idx = 0
 
-        self._maint_commands = Deque[bytes]()
+        self._to_flush = Deque["bytes | None"]()
 
     @staticmethod
     def key(query: PostgresQuery) -> Key:
         return (query.query, query.types)
 
     def get(
-        self, query: PostgresQuery, prepare: Optional[bool] = None
-    ) -> Tuple[Prepare, bytes]:
+        self, query: PostgresQuery, prepare: bool | None = None
+    ) -> tuple[Prepare, bytes]:
         """
         Check if a query is prepared, tell back whether to prepare it.
         """
@@ -77,7 +80,7 @@ class PrepareManager:
             # The query is not to be prepared yet
             return Prepare.NO, b""
 
-    def _should_discard(self, prep: Prepare, results: Sequence["PGresult"]) -> bool:
+    def _should_discard(self, prep: Prepare, results: Sequence[PGresult]) -> bool:
         """Check if we need to discard our entire state: it should happen on
         rollback or on dropping objects, because the same object may get
         recreated and postgres would fail internal lookups.
@@ -92,7 +95,7 @@ class PrepareManager:
         return False
 
     @staticmethod
-    def _check_results(results: Sequence["PGresult"]) -> bool:
+    def _check_results(results: Sequence[PGresult]) -> bool:
         """Return False if 'results' are invalid for prepared statement cache."""
         if len(results) != 1:
             # We cannot prepare a multiple statement
@@ -116,11 +119,11 @@ class PrepareManager:
 
         if len(self._names) > self.prepared_max:
             name = self._names.popitem(last=False)[1]
-            self._maint_commands.append(b"DEALLOCATE " + name)
+            self._to_flush.append(name)
 
     def maybe_add_to_cache(
         self, query: PostgresQuery, prep: Prepare, name: bytes
-    ) -> Optional[Key]:
+    ) -> Key | None:
         """Handle 'query' for possible addition to the cache.
 
         If a new entry has been added, return its key. Return None otherwise
@@ -156,7 +159,7 @@ class PrepareManager:
         key: Key,
         prep: Prepare,
         name: bytes,
-        results: Sequence["PGresult"],
+        results: Sequence[PGresult],
     ) -> None:
         """Validate cached entry with 'key' by checking query 'results'.
 
@@ -180,15 +183,19 @@ class PrepareManager:
         self._counts.clear()
         if self._names:
             self._names.clear()
-            self._maint_commands.clear()
-            self._maint_commands.append(b"DEALLOCATE ALL")
+            self._to_flush.clear()
+            self._to_flush.append(None)
             return True
         else:
             return False
 
-    def get_maintenance_commands(self) -> Iterator[bytes]:
+    def maintain_gen(self, conn: BaseConnection[Any]) -> PQGen[None]:
         """
-        Iterate over the commands needed to align the server state to our state
+        Generator to send the commands to perform periodic maintenance
+
+        Deallocate unneeded command in the server, or flush the prepared
+        statements server state entirely if necessary.
         """
-        while self._maint_commands:
-            yield self._maint_commands.popleft()
+        while self._to_flush:
+            name = self._to_flush.popleft()
+            yield from conn._deallocate(name)
