@@ -1,22 +1,23 @@
 """
-psycopg async cursor objects
+Psycopg AsyncCursor object.
 """
 
 # Copyright (C) 2020 The Psycopg Team
 
+from __future__ import annotations
+
 from types import TracebackType
-from typing import Any, AsyncIterator, Iterable, List
-from typing import Optional, Type, TYPE_CHECKING, overload
+from typing import Any, AsyncIterator, Iterable, TYPE_CHECKING, overload
 from contextlib import asynccontextmanager
 
 from . import pq
 from . import errors as e
 from .abc import Query, Params
-from .copy import AsyncCopy, AsyncWriter as AsyncCopyWriter
+from .copy import AsyncCopy, AsyncWriter
 from .rows import Row, RowMaker, AsyncRowFactory
-from .cursor import BaseCursor
 from ._compat import Self
 from ._pipeline import Pipeline
+from ._cursor_base import BaseCursor
 
 if TYPE_CHECKING:
     from .connection_async import AsyncConnection
@@ -29,20 +30,18 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
     __slots__ = ()
 
     @overload
-    def __init__(self, connection: "AsyncConnection[Row]"):
-        ...
+    def __init__(self, connection: AsyncConnection[Row]): ...
 
     @overload
     def __init__(
-        self, connection: "AsyncConnection[Any]", *, row_factory: AsyncRowFactory[Row]
-    ):
-        ...
+        self, connection: AsyncConnection[Any], *, row_factory: AsyncRowFactory[Row]
+    ): ...
 
     def __init__(
         self,
-        connection: "AsyncConnection[Any]",
+        connection: AsyncConnection[Any],
         *,
-        row_factory: Optional[AsyncRowFactory[Row]] = None,
+        row_factory: AsyncRowFactory[Row] | None = None,
     ):
         super().__init__(connection)
         self._row_factory = row_factory or connection.row_factory
@@ -52,17 +51,21 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
 
     async def __aexit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         await self.close()
 
     async def close(self) -> None:
+        """
+        Close the current cursor and free associated resources.
+        """
         self._close()
 
     @property
     def row_factory(self) -> AsyncRowFactory[Row]:
+        """Writable attribute to control how result rows are formed."""
         return self._row_factory
 
     @row_factory.setter
@@ -77,11 +80,14 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
     async def execute(
         self,
         query: Query,
-        params: Optional[Params] = None,
+        params: Params | None = None,
         *,
-        prepare: Optional[bool] = None,
-        binary: Optional[bool] = None,
+        prepare: bool | None = None,
+        binary: bool | None = None,
     ) -> Self:
+        """
+        Execute a query or command to the database.
+        """
         try:
             async with self._conn.lock:
                 await self._conn.wait(
@@ -98,6 +104,9 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
         *,
         returning: bool = False,
     ) -> None:
+        """
+        Execute the same command with a sequence of input data.
+        """
         try:
             if Pipeline.is_supported():
                 # If there is already a pipeline, ride it, in order to avoid
@@ -125,23 +134,33 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
     async def stream(
         self,
         query: Query,
-        params: Optional[Params] = None,
+        params: Params | None = None,
         *,
-        binary: Optional[bool] = None,
+        binary: bool | None = None,
+        size: int = 1,
     ) -> AsyncIterator[Row]:
+        """
+        Iterate row-by-row on a result from the database.
+
+        :param size: if greater than 1, results will be retrieved by chunks of
+            this size from the server (but still yielded row-by-row); this is only
+            available from version 17 of the libpq.
+        """
         if self._pgconn.pipeline_status:
             raise e.ProgrammingError("stream() cannot be used in pipeline mode")
 
         async with self._conn.lock:
             try:
                 await self._conn.wait(
-                    self._stream_send_gen(query, params, binary=binary)
+                    self._stream_send_gen(query, params, binary=binary, size=size)
                 )
                 first = True
                 while await self._conn.wait(self._stream_fetchone_gen(first)):
-                    # We know that, if we got a result, it has a single row.
-                    rec: Row = self._tx.load_row(0, self._make_row)  # type: ignore
-                    yield rec
+                    for pos in range(size):
+                        rec = self._tx.load_row(pos, self._make_row)
+                        if rec is None:
+                            break
+                        yield rec
                     first = False
 
             except e._NO_TRACEBACK as ex:
@@ -151,7 +170,7 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
                 if self._pgconn.transaction_status == ACTIVE:
                     # Try to cancel the query, then consume the results
                     # already received.
-                    self._conn.cancel()
+                    await self._conn._try_cancel()
                     try:
                         while await self._conn.wait(
                             self._stream_fetchone_gen(first=False)
@@ -167,7 +186,14 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
                     except Exception:
                         pass
 
-    async def fetchone(self) -> Optional[Row]:
+    async def fetchone(self) -> Row | None:
+        """
+        Return the next record from the current recordset.
+
+        Return `!None` the recordset is finished.
+
+        :rtype: Row | None, with Row defined by `row_factory`
+        """
         await self._fetch_pipeline()
         self._check_result_for_fetch()
         record = self._tx.load_row(self._pos, self._make_row)
@@ -175,7 +201,14 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
             self._pos += 1
         return record
 
-    async def fetchmany(self, size: int = 0) -> List[Row]:
+    async def fetchmany(self, size: int = 0) -> list[Row]:
+        """
+        Return the next `!size` records from the current recordset.
+
+        `!size` default to `!self.arraysize` if not specified.
+
+        :rtype: Sequence[Row], with Row defined by `row_factory`
+        """
         await self._fetch_pipeline()
         self._check_result_for_fetch()
         assert self.pgresult
@@ -190,7 +223,12 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
         self._pos += len(records)
         return records
 
-    async def fetchall(self) -> List[Row]:
+    async def fetchall(self) -> list[Row]:
+        """
+        Return all the remaining records from the current recordset.
+
+        :rtype: Sequence[Row], with Row defined by `row_factory`
+        """
         await self._fetch_pipeline()
         self._check_result_for_fetch()
         assert self.pgresult
@@ -202,7 +240,7 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
         await self._fetch_pipeline()
         self._check_result_for_fetch()
 
-        def load(pos: int) -> Optional[Row]:
+        def load(pos: int) -> Row | None:
             return self._tx.load_row(pos, self._make_row)
 
         while True:
@@ -213,6 +251,16 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
             yield row
 
     async def scroll(self, value: int, mode: str = "relative") -> None:
+        """
+        Move the cursor in the result set to a new position according to mode.
+
+        If `!mode` is ``'relative'`` (default), `!value` is taken as offset to
+        the current position in the result set; if set to ``'absolute'``,
+        `!value` states an absolute target position.
+
+        Raise `!IndexError` in case a scroll operation would leave the result
+        set. In this case the position will not change.
+        """
         await self._fetch_pipeline()
         self._scroll(value, mode)
 
@@ -220,12 +268,12 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
     async def copy(
         self,
         statement: Query,
-        params: Optional[Params] = None,
+        params: Params | None = None,
         *,
-        writer: Optional[AsyncCopyWriter] = None,
+        writer: AsyncWriter | None = None,
     ) -> AsyncIterator[AsyncCopy]:
         """
-        :rtype: AsyncCopy
+        Initiate a :sql:`COPY` operation and return an object to manage it.
         """
         try:
             async with self._conn.lock:
@@ -236,6 +284,8 @@ class AsyncCursor(BaseCursor["AsyncConnection[Any]", Row]):
         except e._NO_TRACEBACK as ex:
             raise ex.with_traceback(None)
 
+        # If a fresher result has been set on the cursor by the Copy object,
+        # read its properties (especially rowcount).
         self._select_current_result(0)
 
     async def _fetch_pipeline(self) -> None:
