@@ -7,25 +7,32 @@ the operations, yielding a polling state whenever there is to wait. The
 functions in the `waiting` module are the ones who wait more or less
 cooperatively for the socket to be ready and make these generators continue.
 
-All these generators yield pairs (fileno, `Wait`) whenever an operation would
-block. The generator can be restarted sending the appropriate `Ready` state
-when the file descriptor is ready.
+These generators yield `Wait` objects whenever an operation would block. These
+generators assume the connection fileno will not change. In case of the
+connection function, where the fileno may change, the generators yield pairs
+(fileno, `Wait`).
 
+The generator can be restarted sending the appropriate `Ready` state when the
+file descriptor is ready. If a None value is sent, it means that the wait
+function timed out without any file descriptor becoming ready; in this case the
+generator should probably yield the same value again in order to wait more.
 """
 
 # Copyright (C) 2020 The Psycopg Team
 
+from __future__ import annotations
+
 import logging
-from typing import List, Optional, Union
+from time import monotonic
 
 from . import pq
 from . import errors as e
 from .abc import Buffer, PipelineCommand, PQGen, PQGenConn
-from .pq.abc import PGconn, PGresult
+from .pq.abc import PGcancelConn, PGconn, PGresult
 from .waiting import Wait, Ready
 from ._compat import Deque
 from ._cmodule import _psycopg
-from ._encodings import pgconn_encoding, conninfo_encoding
+from ._encodings import conninfo_encoding
 
 OK = pq.ConnStatus.OK
 BAD = pq.ConnStatus.BAD
@@ -51,31 +58,38 @@ READY_RW = Ready.RW
 logger = logging.getLogger(__name__)
 
 
-def _connect(conninfo: str) -> PQGenConn[PGconn]:
+def _connect(conninfo: str, *, timeout: float = 0.0) -> PQGenConn[PGconn]:
     """
     Generator to create a database connection without blocking.
-
     """
+    deadline = monotonic() + timeout if timeout else 0.0
+
     conn = pq.PGconn.connect_start(conninfo.encode())
     while True:
         if conn.status == BAD:
             encoding = conninfo_encoding(conninfo)
             raise e.OperationalError(
-                f"connection is bad: {pq.error_message(conn, encoding=encoding)}",
+                f"connection is bad: {conn.get_error_message(encoding)}",
                 pgconn=conn,
             )
 
         status = conn.connect_poll()
-        if status == POLL_OK:
+
+        if status == POLL_READING or status == POLL_WRITING:
+            wait = WAIT_R if status == POLL_READING else WAIT_W
+            while True:
+                ready = yield conn.socket, wait
+                if deadline and monotonic() > deadline:
+                    raise e.ConnectionTimeout("connection timeout expired")
+                if ready:
+                    break
+
+        elif status == POLL_OK:
             break
-        elif status == POLL_READING:
-            yield conn.socket, WAIT_R
-        elif status == POLL_WRITING:
-            yield conn.socket, WAIT_W
         elif status == POLL_FAILED:
             encoding = conninfo_encoding(conninfo)
             raise e.OperationalError(
-                f"connection failed: {pq.error_message(conn, encoding=encoding)}",
+                f"connection failed: {conn.get_error_message(encoding)}",
                 pgconn=e.finish_pgconn(conn),
             )
         else:
@@ -87,7 +101,27 @@ def _connect(conninfo: str) -> PQGenConn[PGconn]:
     return conn
 
 
-def _execute(pgconn: PGconn) -> PQGen[List[PGresult]]:
+def _cancel(cancel_conn: PGcancelConn, *, timeout: float = 0.0) -> PQGenConn[None]:
+    deadline = monotonic() + timeout if timeout else 0.0
+    while True:
+        if deadline and monotonic() > deadline:
+            raise e.CancellationTimeout("cancellation timeout expired")
+        status = cancel_conn.poll()
+        if status == POLL_OK:
+            break
+        elif status == POLL_READING:
+            yield cancel_conn.socket, WAIT_R
+        elif status == POLL_WRITING:
+            yield cancel_conn.socket, WAIT_W
+        elif status == POLL_FAILED:
+            raise e.OperationalError(
+                f"cancellation failed: {cancel_conn.get_error_message()}"
+            )
+        else:
+            raise e.InternalError(f"unexpected poll status: {status}")
+
+
+def _execute(pgconn: PGconn) -> PQGen[list[PGresult]]:
     """
     Generator sending a query and returning results without blocking.
 
@@ -119,14 +153,18 @@ def _send(pgconn: PGconn) -> PQGen[None]:
         if f == 0:
             break
 
-        ready = yield WAIT_RW
+        while True:
+            ready = yield WAIT_RW
+            if ready:
+                break
+
         if ready & READY_R:
             # This call may read notifies: they will be saved in the
             # PGconn buffer and passed to Python later, in `fetch()`.
             pgconn.consume_input()
 
 
-def _fetch_many(pgconn: PGconn) -> PQGen[List[PGresult]]:
+def _fetch_many(pgconn: PGconn) -> PQGen[list[PGresult]]:
     """
     Generator retrieving results from the database without blocking.
 
@@ -136,7 +174,7 @@ def _fetch_many(pgconn: PGconn) -> PQGen[List[PGresult]]:
     Return the list of results returned by the database (whether success
     or error).
     """
-    results: List[PGresult] = []
+    results: list[PGresult] = []
     while True:
         res = yield from _fetch(pgconn)
         if not res:
@@ -158,7 +196,7 @@ def _fetch_many(pgconn: PGconn) -> PQGen[List[PGresult]]:
     return results
 
 
-def _fetch(pgconn: PGconn) -> PQGen[Optional[PGresult]]:
+def _fetch(pgconn: PGconn) -> PQGen[PGresult | None]:
     """
     Generator retrieving a single result from the database without blocking.
 
@@ -168,12 +206,19 @@ def _fetch(pgconn: PGconn) -> PQGen[Optional[PGresult]]:
     Return a result from the database (whether success or error).
     """
     if pgconn.is_busy():
-        yield WAIT_R
+        while True:
+            ready = yield WAIT_R
+            if ready:
+                break
+
         while True:
             pgconn.consume_input()
             if not pgconn.is_busy():
                 break
-            yield WAIT_R
+            while True:
+                ready = yield WAIT_R
+                if ready:
+                    break
 
     _consume_notifies(pgconn)
 
@@ -182,7 +227,7 @@ def _fetch(pgconn: PGconn) -> PQGen[Optional[PGresult]]:
 
 def _pipeline_communicate(
     pgconn: PGconn, commands: Deque[PipelineCommand]
-) -> PQGen[List[List[PGresult]]]:
+) -> PQGen[list[list[PGresult]]]:
     """Generator to send queries from a connection in pipeline mode while also
     receiving results.
 
@@ -191,13 +236,16 @@ def _pipeline_communicate(
     results = []
 
     while True:
-        ready = yield WAIT_RW
+        while True:
+            ready = yield WAIT_RW
+            if ready:
+                break
 
         if ready & READY_R:
             pgconn.consume_input()
             _consume_notifies(pgconn)
 
-            res: List[PGresult] = []
+            res: list[PGresult] = []
             while not pgconn.is_busy():
                 r = pgconn.get_result()
                 if r is None:
@@ -241,7 +289,7 @@ def _consume_notifies(pgconn: PGconn) -> None:
             pgconn.notify_handler(n)
 
 
-def notifies(pgconn: PGconn) -> PQGen[List[pq.PGnotify]]:
+def notifies(pgconn: PGconn) -> PQGen[list[pq.PGnotify]]:
     yield WAIT_R
     pgconn.consume_input()
 
@@ -256,14 +304,17 @@ def notifies(pgconn: PGconn) -> PQGen[List[pq.PGnotify]]:
     return ns
 
 
-def copy_from(pgconn: PGconn) -> PQGen[Union[memoryview, PGresult]]:
+def copy_from(pgconn: PGconn) -> PQGen[memoryview | PGresult]:
     while True:
         nbytes, data = pgconn.get_copy_data(1)
         if nbytes != 0:
             break
 
         # would block
-        yield WAIT_R
+        while True:
+            ready = yield WAIT_R
+            if ready:
+                break
         pgconn.consume_input()
 
     if nbytes > 0:
@@ -277,13 +328,12 @@ def copy_from(pgconn: PGconn) -> PQGen[Union[memoryview, PGresult]]:
         raise e.ProgrammingError("you cannot mix COPY with other operations")
     result = results[0]
     if result.status != COMMAND_OK:
-        encoding = pgconn_encoding(pgconn)
-        raise e.error_from_result(result, encoding=encoding)
+        raise e.error_from_result(result, encoding=pgconn._encoding)
 
     return result
 
 
-def copy_to(pgconn: PGconn, buffer: Buffer) -> PQGen[None]:
+def copy_to(pgconn: PGconn, buffer: Buffer, flush: bool = True) -> PQGen[None]:
     # Retry enqueuing data until successful.
     #
     # WARNING! This can cause an infinite loop if the buffer is too large. (see
@@ -291,17 +341,40 @@ def copy_to(pgconn: PGconn, buffer: Buffer) -> PQGen[None]:
     # into smaller ones. We prefer to do it there instead of here in order to
     # do it upstream the queue decoupling the writer task from the producer one.
     while pgconn.put_copy_data(buffer) == 0:
-        yield WAIT_W
+        while True:
+            ready = yield WAIT_W
+            if ready:
+                break
+
+    # Flushing often has a good effect on macOS because memcpy operations
+    # seem expensive on this platform so accumulating a large buffer has a
+    # bad effect (see #745).
+    if flush:
+        # Repeat until it the message is flushed to the server
+        while True:
+            while True:
+                ready = yield WAIT_W
+                if ready:
+                    break
+            f = pgconn.flush()
+            if f == 0:
+                break
 
 
-def copy_end(pgconn: PGconn, error: Optional[bytes]) -> PQGen[PGresult]:
+def copy_end(pgconn: PGconn, error: bytes | None) -> PQGen[PGresult]:
     # Retry enqueuing end copy message until successful
     while pgconn.put_copy_end(error) == 0:
-        yield WAIT_W
+        while True:
+            ready = yield WAIT_W
+            if ready:
+                break
 
     # Repeat until it the message is flushed to the server
     while True:
-        yield WAIT_W
+        while True:
+            ready = yield WAIT_W
+            if ready:
+                break
         f = pgconn.flush()
         if f == 0:
             break
@@ -309,8 +382,7 @@ def copy_end(pgconn: PGconn, error: Optional[bytes]) -> PQGen[PGresult]:
     # Retrieve the final result of copy
     (result,) = yield from _fetch_many(pgconn)
     if result.status != COMMAND_OK:
-        encoding = pgconn_encoding(pgconn)
-        raise e.error_from_result(result, encoding=encoding)
+        raise e.error_from_result(result, encoding=pgconn._encoding)
 
     return result
 
@@ -318,6 +390,7 @@ def copy_end(pgconn: PGconn, error: Optional[bytes]) -> PQGen[PGresult]:
 # Override functions with fast versions if available
 if _psycopg:
     connect = _psycopg.connect
+    cancel = _psycopg.cancel
     execute = _psycopg.execute
     send = _psycopg.send
     fetch_many = _psycopg.fetch_many
@@ -326,6 +399,7 @@ if _psycopg:
 
 else:
     connect = _connect
+    cancel = _cancel
     execute = _execute
     send = _send
     fetch_many = _fetch_many
